"""
BaseInterface Module
====================

This module provides a base interface for defining default classes and options. It includes functionality for setting default parameters, updating interface options, and retrieving updated options.

"""


from .BaseInterface import BaseInterface
from .NonlinearHeat import NonlinearHeat

__all__ = ['BaseInterface', 'NonlinearHeat']